import { Component, Input, TemplateRef, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { HiNGConfig } from '../hi.config';
import { trigger, state, style, animate, transition } from '@angular/animations';
var AutoCompletePopupComponent = (function () {
    function AutoCompletePopupComponent(hiNGConfig) {
        this.activeIndex = 0;
        this.animateState = 'initial';
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.formatter = hiNGConfig.autoComplete.formatter;
    }
    AutoCompletePopupComponent.prototype.writeValue = function (obj) {
        this.value = obj;
    };
    AutoCompletePopupComponent.prototype.show = function () {
        if (!this.isOpen) {
            this.isOpen = true;
            this.animateState = 'visible';
        }
    };
    AutoCompletePopupComponent.prototype.hide = function () {
        if (this.isOpen) {
            this.animateState = 'hidden';
        }
    };
    AutoCompletePopupComponent.prototype.afterVisibilityAnimation = function (e) {
        if (e.toState === 'hidden' && this.isOpen) {
            this.isOpen = false;
        }
    };
    AutoCompletePopupComponent.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
    };
    AutoCompletePopupComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    AutoCompletePopupComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    AutoCompletePopupComponent.prototype.onSelect = function (item) {
        this.value = item;
        this.onTouched();
        this.onChange(this.value);
    };
    AutoCompletePopupComponent.prototype.selectCurrentItem = function () {
        if (this.source && this.source.length) {
            this.onSelect(this.source[this.activeIndex]);
        }
    };
    AutoCompletePopupComponent.prototype.onActiveIndexChange = function (index) {
        this.activeIndex = index;
    };
    AutoCompletePopupComponent.prototype.reset = function () {
        this.activeIndex = 0;
    };
    AutoCompletePopupComponent.prototype.next = function () {
        if (this.isOpen && this.source && this.source.length) {
            if (this.activeIndex === this.source.length - 1) {
                this.activeIndex = 0;
                return;
            }
            this.activeIndex = this.activeIndex + 1;
        }
    };
    AutoCompletePopupComponent.prototype.prev = function () {
        if (this.isOpen && this.source && this.source.length) {
            if (this.activeIndex === 0) {
                this.activeIndex = this.source.length - 1;
                return;
            }
            this.activeIndex = this.activeIndex - 1;
        }
    };
    AutoCompletePopupComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-auto-complete-popup',
                    template: "\n    <ng-template #defaultItemTemplate let-item=\"item\" let-term=\"term\"><hi-hightlight [value]=\"formatter(item)\" [term]=\"term\"></hi-hightlight></ng-template><ul class=\"list-unstyled popup-list\"><li *ngFor=\"let item of source; let $index = index;\" class=\"dropdown-item\" [ngClass]=\"{'bg-primary': $index == activeIndex}\" (click)=\"onSelect(item)\" (mouseenter)=\"onActiveIndexChange($index)\"><ng-template [ngTemplateOutlet]=\"itemTemplate || defaultItemTemplate\" [ngTemplateOutletContext]=\"{ formatter: formatter, term:term, source:source, item:item, $index:$index}\"></ng-template></li><li class=\"dropdown-item\" *ngIf=\"!source?.length && noResultItemTemplate != null\"><ng-template [ngTemplateOutlet]=\"noResultItemTemplate\" [ngTemplateOutletContext]=\"{ term:term, source:source}\"></ng-template></li></ul>\n  ",
                    host: {
                        '[class]': '"dropdown-menu "  +  (cssClass ? cssClass : "")',
                        '[style.display]': 'isOpen && (source?.length || noResultItemTemplate) ? "inline-block" : "none"',
                        '[@state]': 'animateState',
                        '(@state.done)': 'afterVisibilityAnimation($event)'
                    },
                    providers: [{
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return AutoCompletePopupComponent; }),
                            multi: true
                        }],
                    animations: [
                        trigger('state', [
                            state('void', style({ transform: 'scale(0)' })),
                            state('initial', style({ transform: 'scale(0)' })),
                            state('visible', style({ transform: 'scale(1)' })),
                            state('hidden', style({ transform: 'scale(0)' })),
                            transition('* => visible', animate('150ms cubic-bezier(0.0, 0.0, 0.2, 1)')),
                            transition('* => hidden', animate('150ms cubic-bezier(0.4, 0.0, 1, 1)')),
                        ])
                    ]
                },] },
    ];
    AutoCompletePopupComponent.ctorParameters = function () { return [
        { type: HiNGConfig, },
    ]; };
    AutoCompletePopupComponent.propDecorators = {
        "cssClass": [{ type: Input },],
        "disabled": [{ type: Input },],
        "source": [{ type: Input },],
        "isOpen": [{ type: Input },],
        "term": [{ type: Input },],
        "itemTemplate": [{ type: Input },],
        "noResultItemTemplate": [{ type: Input },],
        "formatter": [{ type: Input },],
    };
    return AutoCompletePopupComponent;
}());
export { AutoCompletePopupComponent };
