import { forwardRef, ChangeDetectorRef, Component, ElementRef, EventEmitter, HostListener, Input, Output, TemplateRef, ViewChild, ViewEncapsulation } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { BACKSPACE, DOWN_ARROW, ENTER, ESCAPE, LEFT_ARROW, RIGHT_ARROW, UP_ARROW } from '@angular/cdk/keycodes';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { UpdateHostClassService } from '../common/services/update-host-class.service';
import { dropDownAnimation } from '../common/animation/dropdown-animations';
function toArray(value) {
    var ret;
    if (value == null) {
        ret = [];
    }
    else if (!Array.isArray(value)) {
        ret = [value];
    }
    else {
        ret = value;
    }
    return ret;
}
function arrayEquals(array1, array2) {
    if (!array1 || !array2 || array1.length !== array2.length) {
        return false;
    }
    var len = array1.length;
    for (var i = 0; i < len; i++) {
        if (array1[i] !== array2[i]) {
            return false;
        }
    }
    return true;
}
var defaultDisplayRender = function (label) { return label.join(' / '); };
var ɵ0 = defaultDisplayRender;
var CascaderComponent = (function () {
    function CascaderComponent(elementRef, cdr, updateHostClassService) {
        this.elementRef = elementRef;
        this.cdr = cdr;
        this.updateHostClassService = updateHostClassService;
        this.allowClear = true;
        this.autoFocus = false;
        this.disabled = false;
        this.enableCache = true;
        this.showArrow = true;
        this.showInput = true;
        this.size = 'default';
        this.prefixCls = 'hi-cascader';
        this.inputPrefixCls = 'hi-input';
        this.changeOnSelect = false;
        this.dropDownPosition = 'bottom';
        this.menuVisible = false;
        this.isLoading = false;
        this.isOpening = false;
        this.isFocused = false;
        this.isLabelRenderTemplate = false;
        this.labelRenderContext = {};
        this.selectedOptions = [];
        this.activatedOptions = [];
        this.hiColumns = [];
        this._inputValue = '';
        this.onChange = Function.prototype;
        this.onTouched = Function.prototype;
        this.hiClearText = 'Clear';
        this.hiExpandTrigger = 'click';
        this.hiNotFoundContent = 'Not Found';
        this.hiPlaceHolder = 'Please select';
        this.hiMouseEnterDelay = 150;
        this.hiMouseLeaveDelay = 150;
        this.hiTriggerAction = ['click'];
        this.hiValueProperty = 'value';
        this.hiLabelProperty = 'label';
        this.hiVisibleChange = new EventEmitter();
        this.hiChange = new EventEmitter();
        this.hiSelectionChange = new EventEmitter();
        this.hiSelect = new EventEmitter();
        this.hiClear = new EventEmitter();
        this.el = this.elementRef.nativeElement;
    }
    Object.defineProperty(CascaderComponent.prototype, "inputValue", {
        get: function () {
            return this._inputValue;
        },
        set: function (inputValue) {
            this._inputValue = inputValue;
            this.setClassMap();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CascaderComponent.prototype, "hiLabelRender", {
        get: function () {
            return this.labelRenderTpl;
        },
        set: function (value) {
            this.labelRenderTpl = value;
            this.isLabelRenderTemplate = (value instanceof TemplateRef);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CascaderComponent.prototype, "hiPrefixCls", {
        get: function () {
            return this.prefixCls;
        },
        set: function (prefixCls) {
            this.prefixCls = prefixCls;
            this.setClassMap();
            this.setLabelClass();
            this.setArrowClass();
            this.setLoadingClass();
            this.setClearClass();
            this.setInputClass();
            this.setMenuClass();
            this.setMenuColumnClass();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CascaderComponent.prototype, "hiDisabled", {
        get: function () {
            return this.disabled;
        },
        set: function (value) {
            this.disabled = coerceBooleanProperty(value);
            this.setClassMap();
            this.setInputClass();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CascaderComponent.prototype, "hiSize", {
        get: function () {
            return this.size;
        },
        set: function (value) {
            this.size = value;
            this.setClassMap();
            this.setInputClass();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CascaderComponent.prototype, "hiShowInput", {
        get: function () {
            return this.showInput;
        },
        set: function (value) {
            this.showInput = coerceBooleanProperty(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CascaderComponent.prototype, "hiAllowClear", {
        get: function () {
            return this.allowClear;
        },
        set: function (value) {
            this.allowClear = coerceBooleanProperty(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CascaderComponent.prototype, "hiAutoFocus", {
        get: function () {
            return this.autoFocus;
        },
        set: function (value) {
            this.autoFocus = coerceBooleanProperty(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CascaderComponent.prototype, "hiShowArrow", {
        get: function () {
            return this.showArrow;
        },
        set: function (value) {
            this.showArrow = coerceBooleanProperty(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CascaderComponent.prototype, "hiMenuClassName", {
        get: function () {
            return this.menuClassName;
        },
        set: function (value) {
            this.menuClassName = value;
            this.setMenuClass();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CascaderComponent.prototype, "hiColumnClassName", {
        get: function () {
            return this.columnClassName;
        },
        set: function (value) {
            this.columnClassName = value;
            this.setMenuColumnClass();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CascaderComponent.prototype, "options", {
        get: function () {
            return this.hiColumns[0];
        },
        set: function (options) {
            this.hiColumns = options && options.length ? [options] : [];
            if (this.defaultValue && this.hiColumns.length) {
                this.initOptions(0);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CascaderComponent.prototype, "hiChangeOnSelect", {
        get: function () {
            return this.changeOnSelect;
        },
        set: function (value) {
            this.changeOnSelect = coerceBooleanProperty(value);
        },
        enumerable: true,
        configurable: true
    });
    CascaderComponent.prototype.onPositionChange = function (position) {
        var newValue = position.connectionPair.originY === 'bottom' ? 'bottom' : 'top';
        if (this.dropDownPosition !== newValue) {
            this.dropDownPosition = newValue;
            this.cdr.detectChanges();
        }
    };
    CascaderComponent.prototype.focus = function () {
        if (!this.isFocused) {
            var input = this.el.querySelector("." + this.prefixCls + "-input");
            if (input && input.focus) {
                input.focus();
            }
            else {
                this.el.focus();
            }
            this.isFocused = true;
            this.setClassMap();
        }
    };
    CascaderComponent.prototype.blur = function () {
        if (this.isFocused) {
            var input = this.el.querySelector("." + this.prefixCls + "-input");
            if (input && input.blur) {
                input.blur();
            }
            else {
                this.el.blur();
            }
            this.isFocused = false;
            this.setClassMap();
        }
    };
    CascaderComponent.prototype.setClassMap = function () {
        var classMap = (_a = {},
            _a["" + this.prefixCls] = 1,
            _a[this.prefixCls + "-picker"] = 1,
            _a[this.prefixCls + "-lg"] = this.hiSize === 'large',
            _a[this.prefixCls + "-sm"] = this.hiSize === 'small',
            _a[this.prefixCls + "-picker-disabled"] = this.disabled,
            _a[this.prefixCls + "-focused"] = this.isFocused,
            _a[this.prefixCls + "-picker-open"] = this.menuVisible,
            _a[this.prefixCls + "-picker-with-value"] = this.inputValue && this.inputValue.length,
            _a);
        this.updateHostClassService.updateHostClass(this.el, classMap);
        var _a;
    };
    Object.defineProperty(CascaderComponent.prototype, "labelCls", {
        get: function () {
            return this._labelCls;
        },
        enumerable: true,
        configurable: true
    });
    CascaderComponent.prototype.setLabelClass = function () {
        this._labelCls = (_a = {},
            _a[this.prefixCls + "-picker-label"] = true,
            _a);
        var _a;
    };
    Object.defineProperty(CascaderComponent.prototype, "arrowCls", {
        get: function () {
            return this._arrowCls;
        },
        enumerable: true,
        configurable: true
    });
    CascaderComponent.prototype.setArrowClass = function () {
        this._arrowCls = (_a = {},
            _a[this.prefixCls + "-picker-arrow"] = true,
            _a[this.prefixCls + "-picker-arrow-expand"] = this.menuVisible,
            _a);
        var _a;
    };
    Object.defineProperty(CascaderComponent.prototype, "loadingCls", {
        get: function () {
            return this._loadingCls;
        },
        enumerable: true,
        configurable: true
    });
    CascaderComponent.prototype.setLoadingClass = function () {
        this._loadingCls = (_a = {},
            _a[this.prefixCls + "-picker-arrow"] = true,
            _a);
        var _a;
    };
    Object.defineProperty(CascaderComponent.prototype, "clearCls", {
        get: function () {
            return this._clearCls;
        },
        enumerable: true,
        configurable: true
    });
    CascaderComponent.prototype.setClearClass = function () {
        this._clearCls = (_a = {},
            _a[this.prefixCls + "-picker-clear"] = true,
            _a);
        var _a;
    };
    Object.defineProperty(CascaderComponent.prototype, "inputCls", {
        get: function () {
            return this._inputCls;
        },
        enumerable: true,
        configurable: true
    });
    CascaderComponent.prototype.setInputClass = function () {
        this._inputCls = (_a = {},
            _a[this.prefixCls + "-input"] = 1,
            _a[this.inputPrefixCls + "-disabled"] = this.hiDisabled,
            _a[this.inputPrefixCls + "-lg"] = this.hiSize === 'large',
            _a[this.inputPrefixCls + "-sm"] = this.hiSize === 'small',
            _a);
        var _a;
    };
    Object.defineProperty(CascaderComponent.prototype, "menuCls", {
        get: function () {
            return this._menuCls;
        },
        enumerable: true,
        configurable: true
    });
    CascaderComponent.prototype.setMenuClass = function () {
        this._menuCls = (_a = {},
            _a[this.prefixCls + "-menus"] = true,
            _a[this.prefixCls + "-menus-hidden"] = !this.menuVisible,
            _a["" + this.hiMenuClassName] = this.hiMenuClassName,
            _a);
        var _a;
    };
    Object.defineProperty(CascaderComponent.prototype, "menuColumnCls", {
        get: function () {
            return this._menuColumnCls;
        },
        enumerable: true,
        configurable: true
    });
    CascaderComponent.prototype.setMenuColumnClass = function () {
        this._menuColumnCls = (_a = {},
            _a[this.prefixCls + "-menu"] = true,
            _a["" + this.hiColumnClassName] = this.hiColumnClassName,
            _a);
        var _a;
    };
    CascaderComponent.prototype.getOptionCls = function (option, index) {
        return _a = {},
            _a[this.prefixCls + "-menu-item"] = true,
            _a[this.prefixCls + "-menu-item-expand"] = !option.isLeaf,
            _a[this.prefixCls + "-menu-item-active"] = this.isActivedOption(option, index),
            _a[this.prefixCls + "-menu-item-disabled"] = option.disabled,
            _a[this.prefixCls + "-menu-item-loading"] = option.loading,
            _a;
        var _a;
    };
    CascaderComponent.prototype.handlerInputChange = function (event) {
        event.stopPropagation();
    };
    CascaderComponent.prototype.handleInputBlur = function (event) {
        if (this.menuVisible) {
            this.focus();
        }
        else {
            this.blur();
        }
    };
    CascaderComponent.prototype.handleInputFocus = function (event) {
        this.focus();
    };
    CascaderComponent.prototype.hasInput = function () {
        return this.inputValue.length > 0;
    };
    CascaderComponent.prototype.hasValue = function () {
        return this.value && this.value.length > 0;
    };
    Object.defineProperty(CascaderComponent.prototype, "showPlaceholder", {
        get: function () {
            return !(this.hasInput() || this.hasValue());
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(CascaderComponent.prototype, "showClearIcon", {
        get: function () {
            var isHasValue = this.hasValue();
            var isHasInput = this.hasInput();
            return this.hiAllowClear && !this.hiDisabled && (isHasValue || isHasInput);
        },
        enumerable: true,
        configurable: true
    });
    CascaderComponent.prototype.clearSelection = function (event) {
        if (event) {
            event.preventDefault();
            event.stopPropagation();
        }
        this.labelRenderText = '';
        this.isLabelRenderTemplate = false;
        this.selectedOptions = [];
        this.activatedOptions = [];
        this.inputValue = '';
        this.setMenuVisible(false);
        this.onValueChange();
    };
    CascaderComponent.prototype.buildDisplayLabel = function () {
        var _this = this;
        var selectedOptions = this.selectedOptions;
        var labels = selectedOptions.map(function (o) { return _this.getOptionLabel(o); });
        if (this.isLabelRenderTemplate) {
            this.labelRenderContext = { labels: labels, selectedOptions: selectedOptions };
        }
        else {
            this.labelRenderText = defaultDisplayRender.call(this, labels, selectedOptions);
        }
    };
    CascaderComponent.prototype.onKeyDown = function (event) {
        var keyCode = event.keyCode;
        if (keyCode !== DOWN_ARROW &&
            keyCode !== UP_ARROW &&
            keyCode !== LEFT_ARROW &&
            keyCode !== RIGHT_ARROW &&
            keyCode !== ENTER &&
            keyCode !== BACKSPACE &&
            keyCode !== ESCAPE) {
            return;
        }
        if (!this.isMenuVisible() &&
            keyCode !== BACKSPACE &&
            keyCode !== ESCAPE) {
            this.setMenuVisible(true);
            return;
        }
        if (keyCode === ESCAPE) {
            return;
        }
        if (this.isMenuVisible()) {
            event.preventDefault();
            if (keyCode === DOWN_ARROW) {
                this.moveDown();
            }
            else if (keyCode === UP_ARROW) {
                this.moveUp();
            }
            else if (keyCode === LEFT_ARROW) {
                this.moveLeft();
            }
            else if (keyCode === RIGHT_ARROW) {
                this.moveRight();
            }
            else if (keyCode === ENTER) {
                this.onEnter();
            }
        }
    };
    CascaderComponent.prototype.onTriggerClick = function (event) {
        if (this.hiDisabled) {
            return;
        }
        this.onTouched();
        if (this.isClickTiggerAction()) {
            this.delaySetMenuVisible(!this.menuVisible, 100);
        }
    };
    CascaderComponent.prototype.onTriggerMouseEnter = function (event) {
        if (this.hiDisabled) {
            return;
        }
        if (this.isPointerTiggerAction()) {
            this.delaySetMenuVisible(true, this.hiMouseEnterDelay, true);
        }
    };
    CascaderComponent.prototype.onTriggerMouseLeave = function (event) {
        if (this.hiDisabled) {
            return;
        }
        if (!this.isMenuVisible() || this.isOpening) {
            event.preventDefault();
            return;
        }
        if (this.isPointerTiggerAction()) {
            var mouseTarget = event.relatedTarget;
            var hostEl = this.el;
            var menuEl = this.menu && this.menu.nativeElement;
            if (hostEl.contains(mouseTarget) || (menuEl && menuEl.contains(mouseTarget))) {
                return;
            }
            this.delaySetMenuVisible(false, this.hiMouseLeaveDelay);
        }
    };
    CascaderComponent.prototype.isClickTiggerAction = function () {
        if (typeof this.hiTriggerAction === 'string') {
            return this.hiTriggerAction === 'click';
        }
        return this.hiTriggerAction.indexOf('click') !== -1;
    };
    CascaderComponent.prototype.isPointerTiggerAction = function () {
        if (typeof this.hiTriggerAction === 'string') {
            return this.hiTriggerAction === 'hover';
        }
        return this.hiTriggerAction.indexOf('hover') !== -1;
    };
    CascaderComponent.prototype.closeMenu = function () {
        this.clearDelayTimer();
        this.setMenuVisible(false);
    };
    CascaderComponent.prototype.clearDelayTimer = function () {
        if (this.delayTimer) {
            clearTimeout(this.delayTimer);
            this.delayTimer = null;
        }
    };
    CascaderComponent.prototype.delaySetMenuVisible = function (visible, delay, setOpening) {
        var _this = this;
        if (setOpening === void 0) { setOpening = false; }
        this.clearDelayTimer();
        if (delay) {
            if (visible && setOpening) {
                this.isOpening = true;
            }
            this.delayTimer = setTimeout(function () {
                _this.setMenuVisible(visible);
                _this.clearDelayTimer();
                if (visible) {
                    setTimeout(function () {
                        _this.isOpening = false;
                    }, 100);
                }
            }, delay);
        }
        else {
            this.setMenuVisible(visible);
        }
    };
    CascaderComponent.prototype.isMenuVisible = function () {
        return this.menuVisible;
    };
    CascaderComponent.prototype.setMenuVisible = function (menuVisible) {
        if (this.hiDisabled) {
            return;
        }
        if (this.menuVisible !== menuVisible) {
            this.menuVisible = menuVisible;
            this.setClassMap();
            this.setArrowClass();
            this.setMenuClass();
            if (menuVisible) {
                this.beforeVisible();
            }
            this.hiVisibleChange.emit(menuVisible);
        }
    };
    CascaderComponent.prototype.beforeVisible = function () {
        this.loadRootOptions();
    };
    CascaderComponent.prototype.loadRootOptions = function () {
        if (!this.hiColumns.length) {
            var root = {};
            this.loadChildren(root, -1);
        }
    };
    CascaderComponent.prototype.getOptionLabel = function (option) {
        return option[this.hiLabelProperty || 'label'];
    };
    CascaderComponent.prototype.getOptionValue = function (option) {
        return option[this.hiValueProperty || 'value'];
    };
    CascaderComponent.prototype.isActivedOption = function (option, index) {
        var activeOpt = this.activatedOptions[index];
        return activeOpt === option;
    };
    CascaderComponent.prototype.setActiveOption = function (option, index, select, loadChildren) {
        if (select === void 0) { select = false; }
        if (loadChildren === void 0) { loadChildren = true; }
        if (!option || option.disabled) {
            return;
        }
        this.activatedOptions[index] = option;
        for (var i = index - 1; i >= 0; i--) {
            if (!this.activatedOptions[i]) {
                this.activatedOptions[i] = this.activatedOptions[i + 1].parent;
            }
        }
        if (index < this.activatedOptions.length - 1) {
            this.activatedOptions = this.activatedOptions.slice(0, index + 1);
        }
        if (option.children && option.children.length) {
            option.isLeaf = false;
            option.children.forEach(function (child) { return child.parent = option; });
            this.setColumnData(option.children, index + 1);
        }
        else if (!option.isLeaf && loadChildren) {
            this.loadChildren(option, index);
        }
        else {
            if (index < this.hiColumns.length - 1) {
                this.hiColumns = this.hiColumns.slice(0, index + 1);
            }
        }
        if (select) {
            this.onSelectOption(option, index);
        }
    };
    CascaderComponent.prototype.loadChildren = function (option, index, success, failure) {
        var _this = this;
        if (this.hiLoadData) {
            this.isLoading = index < 0;
            option.loading = true;
            this.hiLoadData(option, index).then(function () {
                option.loading = _this.isLoading = false;
                if (option.children) {
                    option.children.forEach(function (child) { return child.parent = index < 0 ? undefined : option; });
                    _this.setColumnData(option.children, index + 1);
                }
                if (success) {
                    success();
                }
            }, function () {
                option.loading = _this.isLoading = false;
                option.isLeaf = true;
                if (failure) {
                    failure();
                }
            });
        }
    };
    CascaderComponent.prototype.onSelectOption = function (option, index) {
        this.hiSelect.emit({ option: option, index: index });
        if (option.isLeaf || this.hiChangeOnSelect || this.isChangeOn(option, index)) {
            this.selectedOptions = this.activatedOptions;
            this.buildDisplayLabel();
            this.onValueChange();
        }
        if (option.isLeaf) {
            this.delaySetMenuVisible(false, this.hiMouseLeaveDelay);
        }
    };
    CascaderComponent.prototype.isChangeOn = function (option, index) {
        if (typeof this.hiChangeOn === 'function') {
            return this.hiChangeOn(option, index) === true;
        }
        return false;
    };
    CascaderComponent.prototype.setColumnData = function (options, index) {
        if (!arrayEquals(this.hiColumns[index], options)) {
            this.hiColumns[index] = options;
            if (index < this.hiColumns.length - 1) {
                this.hiColumns = this.hiColumns.slice(0, index + 1);
            }
        }
    };
    CascaderComponent.prototype.onOptionClick = function (option, index, event) {
        event.preventDefault();
        this.el.focus();
        if (option && option.disabled) {
            return;
        }
        this.setActiveOption(option, index, true);
    };
    CascaderComponent.prototype.onEnter = function () {
        var columnIndex = Math.max(this.activatedOptions.length - 1, 0);
        var activeOption = this.activatedOptions[columnIndex];
        if (activeOption && !activeOption.disabled) {
            this.onSelectOption(activeOption, columnIndex);
        }
    };
    CascaderComponent.prototype.moveUpOrDown = function (isUp) {
        var columnIndex = Math.max(this.activatedOptions.length - 1, 0);
        var activeOption = this.activatedOptions[columnIndex];
        var options = this.hiColumns[columnIndex] || [];
        var length = options.length;
        var nextIndex = -1;
        if (!activeOption) {
            nextIndex = isUp ? length : -1;
        }
        else {
            nextIndex = options.indexOf(activeOption);
        }
        while (true) {
            nextIndex = isUp ? nextIndex - 1 : nextIndex + 1;
            if (nextIndex < 0 || nextIndex >= length) {
                break;
            }
            var nextOption = options[nextIndex];
            if (!nextOption || nextOption.disabled) {
                continue;
            }
            this.setActiveOption(nextOption, columnIndex);
            break;
        }
    };
    CascaderComponent.prototype.moveUp = function () {
        this.moveUpOrDown(true);
    };
    CascaderComponent.prototype.moveDown = function () {
        this.moveUpOrDown(false);
    };
    CascaderComponent.prototype.moveLeft = function () {
        var options = this.activatedOptions;
        if (options.length) {
            options.pop();
        }
    };
    CascaderComponent.prototype.moveRight = function () {
        var length = this.activatedOptions.length;
        var options = this.hiColumns[length];
        if (options && options.length) {
            var nextOpt = options.find(function (o) { return !o.disabled; });
            if (nextOpt) {
                this.setActiveOption(nextOpt, length);
            }
        }
    };
    CascaderComponent.prototype.onOptionMouseEnter = function (option, index, event) {
        event.preventDefault();
        if (this.hiExpandTrigger === 'hover' && !option.isLeaf) {
            this.delaySelect(option, index, true);
        }
    };
    CascaderComponent.prototype.onOptionMouseLeave = function (option, index, event) {
        event.preventDefault();
        if (this.hiExpandTrigger === 'hover' && !option.isLeaf) {
            this.delaySelect(option, index, false);
        }
    };
    CascaderComponent.prototype.clearDelaySelectTimer = function () {
        if (this.delaySelectTimer) {
            clearTimeout(this.delaySelectTimer);
            this.delaySelectTimer = null;
        }
    };
    CascaderComponent.prototype.delaySelect = function (option, index, doSelect) {
        var _this = this;
        this.clearDelaySelectTimer();
        if (doSelect) {
            this.delaySelectTimer = setTimeout(function () {
                _this.setActiveOption(option, index, true);
                _this.delaySelectTimer = null;
            }, 150);
        }
    };
    CascaderComponent.prototype.getSubmitValue = function () {
        var _this = this;
        var values = [];
        this.selectedOptions.forEach(function (option) {
            values.push(_this.getOptionValue(option));
        });
        return values;
    };
    CascaderComponent.prototype.onValueChange = function () {
        var value = this.getSubmitValue();
        if (!arrayEquals(this.value, value)) {
            this.defaultValue = null;
            this.value = value;
            this.onChange(value);
            if (value.length === 0) {
                this.hiClear.emit();
            }
            this.hiSelectionChange.emit(this.selectedOptions);
            this.hiChange.emit(value);
        }
    };
    CascaderComponent.prototype.findOption = function (option, index) {
        var _this = this;
        var options = this.hiColumns[index];
        if (options) {
            var value_1 = typeof option === 'object' ? this.getOptionValue(option) : option;
            return options.find(function (o) { return value_1 === _this.getOptionValue(o); });
        }
        return null;
    };
    CascaderComponent.prototype.isLoaded = function (index) {
        return this.hiColumns[index] && this.hiColumns[index].length > 0;
    };
    CascaderComponent.prototype.activateOnInit = function (index, value) {
        var option = this.findOption(value, index);
        if (!option) {
            option = typeof value === 'object' ? value : (_a = {},
                _a["" + (this.hiValueProperty || 'value')] = value,
                _a["" + (this.hiLabelProperty || 'label')] = value,
                _a);
        }
        this.setActiveOption(option, index, false, false);
        var _a;
    };
    CascaderComponent.prototype.initOptions = function (index) {
        var _this = this;
        var vs = this.defaultValue;
        var load = function () {
            _this.activateOnInit(index, vs[index]);
            if (index < vs.length - 1) {
                _this.initOptions(index + 1);
            }
            if (index === vs.length - 1) {
                _this.afterWriteValue();
            }
        };
        if (this.isLoaded(index) || !this.hiLoadData) {
            load();
        }
        else {
            var node = this.activatedOptions[index - 1] || {};
            this.loadChildren(node, index - 1, load, this.afterWriteValue);
        }
    };
    CascaderComponent.prototype.afterWriteValue = function () {
        this.selectedOptions = this.activatedOptions;
        this.value = this.getSubmitValue();
        this.buildDisplayLabel();
    };
    CascaderComponent.prototype.writeValue = function (value) {
        var vs = this.defaultValue = toArray(value);
        if (vs.length) {
            this.initOptions(0);
        }
        else {
            this.value = vs;
            this.activatedOptions = [];
            this.afterWriteValue();
        }
    };
    CascaderComponent.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    CascaderComponent.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    CascaderComponent.prototype.setDisabledState = function (isDisabled) {
        if (isDisabled) {
            this.closeMenu();
        }
        this.hiDisabled = isDisabled;
    };
    CascaderComponent.prototype.ngOnInit = function () {
        this.setClassMap();
        this.setLabelClass();
        this.setArrowClass();
        this.setLoadingClass();
        this.setClearClass();
        this.setInputClass();
        this.setMenuClass();
        this.setMenuColumnClass();
    };
    CascaderComponent.prototype.ngOnDestroy = function () {
        this.clearDelayTimer();
        this.clearDelaySelectTimer();
    };
    CascaderComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-cascader,[hi-cascader]',
                    preserveWhitespaces: false,
                    animations: [
                        dropDownAnimation
                    ],
                    template: "\n    <div cdkOverlayOrigin #origin=\"cdkOverlayOrigin\" #trigger><div *ngIf=\"hiShowInput\"><input #input hi-input [attr.autoComplete]=\"'off'\" [attr.placeholder]=\"showPlaceholder ? hiPlaceHolder : null\" [attr.autofocus]=\"hiAutoFocus ? 'autofocus' : null\" [readonly]=\"!hiShowSearch\" [disabled]=\"hiDisabled\" [hiSize]=\"hiSize\" [ngClass]=\"inputCls\" [(ngModel)]=\"inputValue\" (blur)=\"handleInputBlur($event)\" (focus)=\"handleInputFocus($event)\" (change)=\"handlerInputChange($event)\"> <i *ngIf=\"showClearIcon\" [class]=\"'anticon anticon-cross-circle'\" [ngClass]=\"clearCls\" [attr.title]=\"hiClearText\" (click)=\"clearSelection($event)\"></i> <i *ngIf=\"hiShowArrow && !isLoading\" class=\"anticon anticon-down\" [ngClass]=\"arrowCls\"></i> <i *ngIf=\"isLoading\" class=\"anticon anticon-loading anticon-spin\" [ngClass]=\"loadingCls\"></i> <span [ngClass]=\"labelCls\"><ng-container *ngIf=\"!isLabelRenderTemplate; else labelTemplate\">{{ labelRenderText }}</ng-container><ng-template #labelTemplate><ng-template [ngTemplateOutlet]=\"hiLabelRender\" [ngTemplateOutletContext]=\"labelRenderContext\"></ng-template></ng-template></span></div><ng-content></ng-content></div><ng-template cdkConnectedOverlay cdkConnectedOverlayHasBackdrop [cdkConnectedOverlayOrigin]=\"origin\" (backdropClick)=\"closeMenu()\" (detach)=\"closeMenu()\" (positionChange)=\"onPositionChange($event)\" [cdkConnectedOverlayOpen]=\"menuVisible\"><div #menu [ngClass]=\"menuCls\" [ngStyle]=\"hiMenuStyle\" [@dropDownAnimation]=\"dropDownPosition\" (mouseleave)=\"onTriggerMouseLeave($event)\"><ul *ngFor=\"let options of hiColumns; let i = index;\" [ngClass]=\"menuColumnCls\"><li *ngFor=\"let option of options\" [attr.title]=\"option.title || getOptionLabel(option)\" [ngClass]=\"getOptionCls(option, i)\" (mouseenter)=\"onOptionMouseEnter(option, i, $event)\" (mouseleave)=\"onOptionMouseLeave(option, i, $event)\" (click)=\"onOptionClick(option, i, $event)\">{{ getOptionLabel(option) }}</li></ul></div></ng-template>\n  ",
                    styles: ["\n    .cdk-overlay-container{pointer-events:none;top:0;left:0;height:100%;width:100%;position:fixed;z-index:1000}.cdk-overlay-backdrop{top:0;bottom:0;left:0;right:0;-webkit-tap-highlight-color:transparent;-webkit-transition:opacity .4s cubic-bezier(.25,.8,.25,1);transition:opacity .4s cubic-bezier(.25,.8,.25,1);opacity:0}.cdk-overlay-backdrop,.cdk-overlay-pane{position:absolute;pointer-events:auto;z-index:1000}@font-face{font-family:anticon;src:url(https://at.alicdn.com/t/font_zck90zmlh7hf47vi.eot);src:url(https://at.alicdn.com/t/font_zck90zmlh7hf47vi.eot#iefix) format(\"embedded-opentype\"),url(https://at.alicdn.com/t/font_zck90zmlh7hf47vi.woff) format(\"woff\"),url(https://at.alicdn.com/t/font_zck90zmlh7hf47vi.ttf) format(\"truetype\"),url(https://at.alicdn.com/t/font_zck90zmlh7hf47vi.svg#iconfont) format(\"svg\")}.anticon{display:inline-block;font-style:normal;vertical-align:baseline;text-align:center;text-transform:none;line-height:1;text-rendering:optimizeLegibility;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}.anticon:before{display:block;font-family:anticon!important}.anticon-close-circle:before,.anticon-cross-circle:before{content:\"\\E62E\"}.anticon:before{display:block;font-family:anticon!important}.anticon-down:before{content:\"\\E61D\"}.anticon-up:before{content:\"\\E61E\"}.anticon-right:before{content:\"\\E61F\"}.anticon-left:before{content:\"\\E620\"}.hi-cascader{display:block}.hi-cascader-menus{top:100%;left:0;position:relative;width:100%;margin-top:4px;margin-bottom:4px}.hi-cascader{font-size:12px}.hi-cascader-input.hi-input{background-color:transparent!important;cursor:pointer;width:100%;display:block}.hi-input{position:relative;display:inline-block;padding:4px 7px;width:100%;height:28px;font-size:12px;line-height:1.5;color:rgba(0,0,0,.65);background-color:#fff;background-image:none;border:1px solid #d9d9d9;border-radius:4px;-webkit-transition:all .3s;transition:all .3s}.hi-cascader-picker{position:relative;display:inline-block;cursor:pointer;font-size:12px;background-color:#fff;border-radius:4px;outline:0}.hi-cascader-picker-with-value .hi-cascader-picker-label{color:transparent}.hi-cascader-picker-disabled{cursor:not-allowed;background:#f7f7f7;color:rgba(0,0,0,.25)}.hi-cascader-picker-disabled .hi-cascader-input{cursor:not-allowed}.hi-cascader-picker:focus .hi-cascader-input{border-color:#28a745;outline:0;-webkit-box-shadow:0 0 0 2px rgba(16,142,233,.2);box-shadow:0 0 0 2px rgba(16,142,233,.2)}.hi-cascader-picker-label{position:absolute;left:0;height:20px;line-height:20px;top:50%;margin-top:-10px;white-space:nowrap;text-overflow:ellipsis;overflow:hidden;width:100%;padding:0 12px 0 8px}.hi-cascader-picker-clear{opacity:0;position:absolute;right:8px;z-index:2;background:#fff;top:50%;font-size:12px;color:rgba(0,0,0,.25);width:12px;height:12px;margin-top:-6px;line-height:12px;cursor:pointer;-webkit-transition:color .3s ease,opacity .15s ease;transition:color .3s ease,opacity .15s ease}.hi-cascader-picker-clear:hover{color:rgba(0,0,0,.43)}.hi-cascader-picker:hover .hi-cascader-picker-clear{opacity:1}.hi-cascader-picker-arrow{position:absolute;z-index:1;top:50%;right:8px;width:12px;height:12px;margin-top:-6px;line-height:12px;color:rgba(0,0,0,.43);display:inline-block;font-size:12px;font-size:9px\\9;-webkit-transform:scale(.75) rotate(0);transform:scale(.75) rotate(0);-ms-filter:\"progid:DXImageTransform.Microsoft.Matrix(sizingMethod='auto expand', M11=1, M12=0, M21=0, M22=1)\";zoom:1}.hi-cascader-picker-arrow{-webkit-filter:none;filter:none;font-size:12px}.hi-cascader-picker-arrow:before{-webkit-transition:-webkit-transform .2s ease;transition:-webkit-transform .2s ease;transition:transform .2s ease;transition:transform .2s ease,-webkit-transform .2s ease}.hi-cascader-picker-arrow.hi-cascader-picker-arrow-expand{-ms-filter:\"progid:DXImageTransform.Microsoft.BasicImage(rotation=2)\"}.hi-cascader-picker-arrow.hi-cascader-picker-arrow-expand:before{-webkit-transform:rotate(180deg);transform:rotate(180deg)}.hi-cascader-menus{font-size:12px;background:#fff;position:absolute;z-index:1050;border-radius:4px;-webkit-box-shadow:0 1px 6px rgba(0,0,0,.2);box-shadow:0 1px 6px rgba(0,0,0,.2);white-space:nowrap}.hi-cascader-menus-empty,.hi-cascader-menus-hidden{display:none}.hi-cascader-menus.slide-up-appear.slide-up-appear-active.hi-cascader-menus-placement-bottomLeft,.hi-cascader-menus.slide-up-enter.slide-up-enter-active.hi-cascader-menus-placement-bottomLeft{-webkit-animation-name:antSlideUpIn;animation-name:antSlideUpIn}.hi-cascader-menus.slide-up-appear.slide-up-appear-active.hi-cascader-menus-placement-topLeft,.hi-cascader-menus.slide-up-enter.slide-up-enter-active.hi-cascader-menus-placement-topLeft{-webkit-animation-name:antSlideDownIn;animation-name:antSlideDownIn}.hi-cascader-menus.slide-up-leave.slide-up-leave-active.hi-cascader-menus-placement-bottomLeft{-webkit-animation-name:antSlideUpOut;animation-name:antSlideUpOut}.hi-cascader-menus.slide-up-leave.slide-up-leave-active.hi-cascader-menus-placement-topLeft{-webkit-animation-name:antSlideDownOut;animation-name:antSlideDownOut}.hi-cascader-menu{display:inline-block;vertical-align:top;min-width:111px;height:180px;list-style:none;margin:0;padding:0;border-right:1px solid #e9e9e9;overflow:auto}.hi-cascader-menu:first-child{border-radius:4px 0 0 4px}.hi-cascader-menu:last-child{border-right-color:transparent;margin-right:-1px;border-radius:0 4px 4px 0}.hi-cascader-menu:only-child{border-radius:4px}.hi-cascader-menu-item{padding:7px 8px;cursor:pointer;white-space:nowrap;-webkit-transition:all .3s;transition:all .3s}.hi-cascader-menu-item:hover{background:#e6fde8}.hi-cascader-menu-item-disabled{cursor:not-allowed;color:rgba(0,0,0,.25)}.hi-cascader-menu-item-disabled:hover{background:0 0}.hi-cascader-menu-item-active:not(.hi-cascader-menu-item-disabled),.hi-cascader-menu-item-active:not(.hi-cascader-menu-item-disabled):hover{background:#f7f7f7;font-weight:600}.hi-cascader-menu-item-expand{position:relative;padding-right:24px}.hi-cascader-menu-item-expand:after{font-family:anticon;text-rendering:optimizeLegibility;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale;content:\"\\E61F\";display:inline-block;font-size:12px;font-size:8px\\9;-webkit-transform:scale(.66667) rotate(0);transform:scale(.66667) rotate(0);-ms-filter:\"progid:DXImageTransform.Microsoft.Matrix(sizingMethod='auto expand', M11=1, M12=0, M21=0, M22=1)\";zoom:1;color:rgba(0,0,0,.43);position:absolute;right:8px}.hi-cascader-menu-item-expand:after{-webkit-filter:none;filter:none;font-size:12px}.hi-cascader-menu-item-expand:after,.hi-cascader-menu-item-expand:before{-webkit-box-sizing:border-box;box-sizing:border-box}.hi-cascader-menu-item-loading:after{font-family:anticon;text-rendering:optimizeLegibility;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale;content:\"\\E64D\";-webkit-animation:loadingCircle 1s infinite linear;animation:loadingCircle 1s infinite linear}.hi-cascader-menu-item .hi-cascader-menu-item-keyword{color:#f04134}.hi-cascader-menus{top:100%;left:0;position:relative;width:100%;margin-top:4px;margin-bottom:4px}\n  "],
                    providers: [
                        UpdateHostClassService,
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return CascaderComponent; }),
                            multi: true
                        }
                    ],
                    host: {
                        '[attr.tabIndex]': '"0"',
                    },
                    encapsulation: ViewEncapsulation.None,
                },] },
    ];
    CascaderComponent.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: ChangeDetectorRef, },
        { type: UpdateHostClassService, },
    ]; };
    CascaderComponent.propDecorators = {
        "hiLabelRender": [{ type: Input },],
        "hiPrefixCls": [{ type: Input },],
        "hiDisabled": [{ type: Input },],
        "hiSize": [{ type: Input },],
        "hiShowInput": [{ type: Input },],
        "hiAllowClear": [{ type: Input },],
        "hiAutoFocus": [{ type: Input },],
        "hiShowArrow": [{ type: Input },],
        "hiMenuClassName": [{ type: Input },],
        "hiColumnClassName": [{ type: Input },],
        "options": [{ type: Input },],
        "hiChangeOnSelect": [{ type: Input },],
        "hiClearText": [{ type: Input },],
        "hiExpandTrigger": [{ type: Input },],
        "hiNotFoundContent": [{ type: Input },],
        "hiPlaceHolder": [{ type: Input },],
        "hiMenuStyle": [{ type: Input },],
        "hiChangeOn": [{ type: Input },],
        "hiMouseEnterDelay": [{ type: Input },],
        "hiMouseLeaveDelay": [{ type: Input },],
        "hiTriggerAction": [{ type: Input },],
        "hiValueProperty": [{ type: Input },],
        "hiLabelProperty": [{ type: Input },],
        "hiLoadData": [{ type: Input },],
        "hiVisibleChange": [{ type: Output },],
        "hiChange": [{ type: Output },],
        "hiSelectionChange": [{ type: Output },],
        "hiSelect": [{ type: Output },],
        "hiClear": [{ type: Output },],
        "menu": [{ type: ViewChild, args: ['menu',] },],
        "onKeyDown": [{ type: HostListener, args: ['keydown', ['$event'],] },],
        "onTriggerClick": [{ type: HostListener, args: ['click', ['$event'],] },],
        "onTriggerMouseEnter": [{ type: HostListener, args: ['mouseenter', ['$event'],] },],
        "onTriggerMouseLeave": [{ type: HostListener, args: ['mouseleave', ['$event'],] },],
    };
    return CascaderComponent;
}());
export { CascaderComponent };
export { ɵ0 };
