import { Component, ChangeDetectionStrategy, Input, Output, EventEmitter, HostListener, ElementRef, TemplateRef } from '@angular/core';
import { WindowRef } from '../window-ref/window-ref.service';
var InfiniteScrollComponent = (function () {
    function InfiniteScrollComponent(windowRef, elementRef) {
        this.windowRef = windowRef;
        this.elementRef = elementRef;
        this.bufferPx = 40;
        this.scrollDelay = 300;
        this.loadMore = new EventEmitter();
        this.scrollStream = new EventEmitter();
    }
    InfiniteScrollComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.setupScrollEvent();
        setTimeout(function (_) { return _this.onWinScroll(null); });
    };
    InfiniteScrollComponent.prototype.loadFinish = function (complete) {
        var _this = this;
        if (complete) {
            return this.unSubscription();
        }
        setTimeout(function (_) { return _this.onWinScroll(null); }, this.scrollDelay);
    };
    InfiniteScrollComponent.prototype.setupScrollEvent = function () {
        var _this = this;
        if (!this.subscription) {
            this.subscription = this.registerOnScrollStream(this.scrollStream)
                .subscribe(function (_) { return _this.onScrollChange(); });
        }
    };
    InfiniteScrollComponent.prototype.onScrollChange = function () {
        var winHeight = this.windowRef.innerHeight;
        var clientRect = this.windowRef.getBoundingClientRect(this.elementRef);
        if (clientRect && winHeight - clientRect.top >= this.bufferPx) {
            this.loadMore.emit(this);
        }
    };
    InfiniteScrollComponent.prototype.onWinScroll = function ($event) {
        this.scrollStream.emit($event);
    };
    InfiniteScrollComponent.prototype.ngOnDestroy = function () {
        this.unSubscription();
    };
    InfiniteScrollComponent.prototype.unSubscription = function () {
        if (this.subscription) {
            this.subscription.unsubscribe();
            this.subscription = null;
        }
    };
    InfiniteScrollComponent.prototype.registerOnScrollStream = function (scrollStream) {
        return scrollStream
            .debounceTime(this.scrollDelay)
            .distinctUntilChanged();
    };
    InfiniteScrollComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-infinite-scroll',
                    template: "\n    <ng-template #defaultTemplate let-spinner=\"\"><div class=\"spinner\"><div class=\"bounce bounce1\"></div><div class=\"bounce bounce2\"></div><div class=\"bounce bounce3\"></div></div></ng-template><ng-template [ngTemplateOutlet]=\"spinnerTemplate || defaultTemplate\" [ngTemplateOutletContext]=\"{ $implicit: this }\"></ng-template>\n  ",
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    exportAs: 'infiniteScroll'
                },] },
    ];
    InfiniteScrollComponent.ctorParameters = function () { return [
        { type: WindowRef, },
        { type: ElementRef, },
    ]; };
    InfiniteScrollComponent.propDecorators = {
        "total": [{ type: Input },],
        "bufferPx": [{ type: Input },],
        "scrollDelay": [{ type: Input },],
        "spinnerTemplate": [{ type: Input },],
        "loadMore": [{ type: Output },],
        "onWinScroll": [{ type: HostListener, args: ['window:scroll', ['$event'],] },],
    };
    return InfiniteScrollComponent;
}());
export { InfiniteScrollComponent };
