import { Directive, ElementRef, HostBinding, HostListener, Input, Optional, Renderer2, Self } from '@angular/core';
import { NgControl, NgModel } from '@angular/forms';
import calculateNodeHeight from '../utils/calculate-node-height';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
var InputDirective = (function () {
    function InputDirective(elementRef, renderer, ngModel, ngControl) {
        this.elementRef = elementRef;
        this.renderer = renderer;
        this.ngModel = ngModel;
        this.ngControl = ngControl;
        this._size = 'default';
        this._disabled = false;
        this._autosize = false;
        this.isInit = false;
        this.el = this.elementRef.nativeElement;
    }
    Object.defineProperty(InputDirective.prototype, "hiSize", {
        get: function () {
            return this._size;
        },
        set: function (value) {
            this._size = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputDirective.prototype, "disabled", {
        get: function () {
            if (this.ngControl && this.ngControl.disabled !== null) {
                return this.ngControl.disabled;
            }
            return this._disabled;
        },
        set: function (value) {
            this._disabled = coerceBooleanProperty(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputDirective.prototype, "hiAutosize", {
        get: function () {
            return this._autosize;
        },
        set: function (value) {
            if (typeof value === 'string') {
                this._autosize = true;
            }
            else {
                this._autosize = value;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputDirective.prototype, "setLgClass", {
        get: function () {
            return this.hiSize === 'large';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(InputDirective.prototype, "setSmClass", {
        get: function () {
            return this.hiSize === 'small';
        },
        enumerable: true,
        configurable: true
    });
    InputDirective.prototype.textAreaOnChange = function () {
        if (this.hiAutosize) {
            this.resizeTextArea();
        }
    };
    InputDirective.prototype.resizeTextArea = function () {
        var textAreaRef = this.el;
        var maxRows = this.hiAutosize ? this.hiAutosize.maxRows || null : null;
        var minRows = this.hiAutosize ? this.hiAutosize.minRows || null : null;
        if ((this.previousValue === textAreaRef.value) && (this.previewsMaxRows === maxRows) && (this.previewsMinRows === minRows)) {
            return;
        }
        this.previousValue = textAreaRef.value;
        this.previewsMinRows = minRows;
        this.previewsMaxRows = maxRows;
        this.renderer.setStyle(textAreaRef, 'height', 'auto');
        var textAreaStyles = calculateNodeHeight(textAreaRef, false, minRows, maxRows);
        this.renderer.setStyle(textAreaRef, 'height', textAreaStyles.height + "px");
        this.renderer.setStyle(textAreaRef, 'overflowY', textAreaStyles.overflowY);
        this.renderer.setStyle(textAreaRef, 'minHeight', textAreaStyles.minHeight + "px");
        this.renderer.setStyle(textAreaRef, 'maxHeight', textAreaStyles.maxHeight + "px");
    };
    InputDirective.prototype.ngDoCheck = function () {
        if (this.hiAutosize && this.isInit) {
            this.resizeTextArea();
        }
    };
    InputDirective.prototype.ngAfterViewInit = function () {
        this.isInit = true;
        if (this.hiAutosize) {
            this.resizeTextArea();
        }
    };
    InputDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[hi-input]',
                    host: {
                        '[class.hi-input]': 'true'
                    }
                },] },
    ];
    InputDirective.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: Renderer2, },
        { type: NgModel, decorators: [{ type: Optional },] },
        { type: NgControl, decorators: [{ type: Optional }, { type: Self },] },
    ]; };
    InputDirective.propDecorators = {
        "hiSize": [{ type: Input },],
        "disabled": [{ type: Input }, { type: HostBinding, args: ["class.hi-input-disabled",] },],
        "hiAutosize": [{ type: Input },],
        "setLgClass": [{ type: HostBinding, args: ["class.hi-input-lg",] },],
        "setSmClass": [{ type: HostBinding, args: ["class.hi-input-sm",] },],
        "textAreaOnChange": [{ type: HostListener, args: ['input',] },],
    };
    return InputDirective;
}());
export { InputDirective };
