import { ComponentFactoryResolver, Injectable, Injector } from '@angular/core';
import { HiNGConfig } from '../hi.config';
import { OverlayComponent } from './overlay.component';
import { DomSanitizer } from '@angular/platform-browser';
var OverlayService = (function () {
    function OverlayService(hiNGConfig, componentFactoryResolver, injector, domSanitizer) {
        this.hiNGConfig = hiNGConfig;
        this.componentFactoryResolver = componentFactoryResolver;
        this.injector = injector;
        this.domSanitizer = domSanitizer;
        this.overlayCount = 0;
    }
    OverlayService.prototype.open = function (options) {
        if (!this.overlayCount++) {
            this.createOverlay(options);
        }
    };
    OverlayService.prototype.createOverlay = function (options) {
        var rootContainer = options.rootContainer || this.hiNGConfig.rootContainer;
        if (!rootContainer) {
            throw new Error('Should setup ViewContainerRef on modal overlay or hi config service!');
        }
        if (options.html && (typeof options.html === 'string')) {
            options.html = this.domSanitizer.bypassSecurityTrustHtml(options.html);
        }
        var componentFactory = this.componentFactoryResolver.resolveComponentFactory(OverlayComponent);
        var injector = options.injector || this.injector;
        this.overlayRef = rootContainer.createComponent(componentFactory, rootContainer.length, injector);
        var cmp = this.overlayRef.instance;
        cmp.addContent(options);
    };
    OverlayService.prototype.close = function () {
        this.overlayCount = this.overlayCount > 0 ? this.overlayCount - 1 : 0;
        if (this.overlayCount < 1 && this.overlayRef) {
            this.overlayRef.destroy();
        }
    };
    OverlayService.decorators = [
        { type: Injectable },
    ];
    OverlayService.ctorParameters = function () { return [
        { type: HiNGConfig, },
        { type: ComponentFactoryResolver, },
        { type: Injector, },
        { type: DomSanitizer, },
    ]; };
    return OverlayService;
}());
export { OverlayService };
