import { Component, ChangeDetectionStrategy, Input, Output, EventEmitter } from '@angular/core';
import { HiNGConfig } from '../hi.config';
var PagerComponent = (function () {
    function PagerComponent(hiNGConfig) {
        this.hiNGConfig = hiNGConfig;
        this.total = 0;
        this.pageIndex = 1;
        this.pageIndexChange = new EventEmitter();
        this.disable = false;
        this.totalPage = 0;
        this.aligned = hiNGConfig.pager.aligned;
        this.pageSize = hiNGConfig.pager.pageSize;
        this.previousText = hiNGConfig.pager.button.previous;
        this.nextText = hiNGConfig.pager.button.next;
    }
    PagerComponent.prototype.prev = function () {
        if (this.hasPrev()) {
            this.onPageIndexChange(this.pageIndex - 1);
        }
    };
    PagerComponent.prototype.next = function () {
        if (this.hasNext()) {
            this.onPageIndexChange(this.pageIndex + 1);
        }
    };
    PagerComponent.prototype.onPageIndexChange = function (pageIndex) {
        if (this.pageIndex !== pageIndex) {
            this.pageIndexChange.emit(pageIndex);
        }
    };
    PagerComponent.prototype.hasPrev = function () {
        return this.pageIndex > 1;
    };
    PagerComponent.prototype.hasNext = function () {
        return this.pageIndex < this.totalPage;
    };
    PagerComponent.prototype.getTotalPage = function () {
        return Math.ceil(this.total / this.pageSize);
    };
    PagerComponent.prototype.ngOnChanges = function (changes) {
        var shouldUpdateRanges = PagerComponent.EFFECT_PAGE_RANGE_KEYS.some(function (key) { return !!changes[key]; });
        if (shouldUpdateRanges) {
            this.totalPage = this.getTotalPage();
            this.pageIndex = Math.max(Math.min(this.pageIndex, this.totalPage), 1);
        }
    };
    PagerComponent.EFFECT_PAGE_RANGE_KEYS = ['total', 'pageSize', 'pageIndex'];
    PagerComponent.decorators = [
        { type: Component, args: [{
                    selector: 'hi-pager',
                    template: "\n    <nav aria-label=\"pager\" class=\"{{cssClass}}\"><ul class=\"pager\"><li [ngClass]=\"{'previous': aligned, 'disabled': disable && !hasPrev()}\" *ngIf=\"disable || hasPrev()\"><a (click)=\"prev()\">{{ previousText}}</a></li><li [ngClass]=\"{'next': aligned, 'disabled': disable && !hasNext()}\" *ngIf=\"disable || hasNext()\"><a (click)=\"next()\">{{ nextText}}</a></li></ul></nav>\n  ",
                    exportAs: 'pager',
                    changeDetection: ChangeDetectionStrategy.OnPush
                },] },
    ];
    PagerComponent.ctorParameters = function () { return [
        { type: HiNGConfig, },
    ]; };
    PagerComponent.propDecorators = {
        "total": [{ type: Input },],
        "pageSize": [{ type: Input },],
        "pageIndex": [{ type: Input },],
        "pageIndexChange": [{ type: Output },],
        "aligned": [{ type: Input },],
        "previousText": [{ type: Input },],
        "nextText": [{ type: Input },],
        "disable": [{ type: Input },],
        "cssClass": [{ type: Input },],
    };
    return PagerComponent;
}());
export { PagerComponent };
