import { fromPromise } from 'rxjs/observable/fromPromise';
import { Injectable } from '@angular/core';
function cloneDeep(obj) {
    return JSON.parse(JSON.stringify(obj));
}
export var DEFAULT_STORAGE_POOL_KEY = 'storage:default';
export var StorageType;
(function (StorageType) {
    StorageType[StorageType["memory"] = 0] = "memory";
    StorageType[StorageType["sessionStorage"] = 1] = "sessionStorage";
    StorageType[StorageType["localStorage"] = 2] = "localStorage";
})(StorageType || (StorageType = {}));
var RxDataCacheStrategy = (function () {
    function RxDataCacheStrategy() {
    }
    RxDataCacheStrategy.prototype.name = function () {
        return 'RxDataCacheStrategy';
    };
    RxDataCacheStrategy.prototype.match = function (result) {
        return result && result.subscribe;
    };
    RxDataCacheStrategy.prototype.put = function (result, putStorage) {
        return result.map(function (data) {
            putStorage(data);
            return data;
        });
    };
    RxDataCacheStrategy.prototype.get = function (result) {
        return fromPromise(Promise.resolve(result));
    };
    return RxDataCacheStrategy;
}());
var PromiseDataCacheStrategy = (function () {
    function PromiseDataCacheStrategy() {
    }
    PromiseDataCacheStrategy.prototype.name = function () {
        return 'PromiseDataCacheStrategy';
    };
    PromiseDataCacheStrategy.prototype.match = function (result) {
        return result && result.then;
    };
    PromiseDataCacheStrategy.prototype.put = function (result, putStorage) {
        return result.then(function (data) { return putStorage(data); });
    };
    PromiseDataCacheStrategy.prototype.get = function (result) {
        return Promise.resolve(result);
    };
    return PromiseDataCacheStrategy;
}());
var DataCacheStrategyFactory = (function () {
    function DataCacheStrategyFactory() {
        this.dataCacheStrategies = [new RxDataCacheStrategy(), new PromiseDataCacheStrategy()];
    }
    DataCacheStrategyFactory.getInstance = function () {
        return DataCacheStrategyFactory.factory;
    };
    DataCacheStrategyFactory.prototype.put = function (options, value, storage) {
        var strategy = this.dataCacheStrategies.find(function (t) { return t.match(value); });
        if (strategy) {
            return strategy.put(value, function (result) { return storage.put(options, { type: strategy.name(), result: result }); });
        }
        storage.put(options, value);
        return value;
    };
    DataCacheStrategyFactory.prototype.get = function (data) {
        if (data && data.type) {
            var strategy = this.dataCacheStrategies.find(function (t) { return t.name() === data.type; });
            if (strategy) {
                return strategy.get(data.result);
            }
        }
        return data;
    };
    DataCacheStrategyFactory.factory = new DataCacheStrategyFactory();
    return DataCacheStrategyFactory;
}());
var WebStorage = (function () {
    function WebStorage(webStorage) {
        this.webStorage = webStorage;
    }
    WebStorage.prototype.getAll = function (pool) {
        var json = this.webStorage.getItem(pool);
        return json ? JSON.parse(json) : {};
    };
    WebStorage.prototype.saveAll = function (pool, storage) {
        this.webStorage.setItem(pool, JSON.stringify(storage));
    };
    WebStorage.prototype.get = function (_a) {
        var _b = _a.pool, pool = _b === void 0 ? DEFAULT_STORAGE_POOL_KEY : _b, key = _a.key;
        var storage = this.getAll(pool);
        return storage[key];
    };
    WebStorage.prototype.put = function (_a, value) {
        var _b = _a.pool, pool = _b === void 0 ? DEFAULT_STORAGE_POOL_KEY : _b, key = _a.key;
        var storage = this.getAll(pool);
        storage[key] = value;
        return this.saveAll(pool, storage);
    };
    WebStorage.prototype.remove = function (_a) {
        var _b = _a.pool, pool = _b === void 0 ? DEFAULT_STORAGE_POOL_KEY : _b, key = _a.key;
        if (!key) {
            this.webStorage.removeItem(pool);
            return;
        }
        this.put({ pool: pool, key: key }, null);
    };
    WebStorage.prototype.removeAll = function () {
        this.webStorage.clear();
    };
    return WebStorage;
}());
export { WebStorage };
var MemoryStorage = (function () {
    function MemoryStorage() {
        this.storage = new Map();
    }
    MemoryStorage.prototype.getAll = function (pool) {
        return this.storage.has(pool) ? this.storage.get(pool) : new Map();
    };
    MemoryStorage.prototype.get = function (_a) {
        var _b = _a.pool, pool = _b === void 0 ? DEFAULT_STORAGE_POOL_KEY : _b, key = _a.key;
        var storage = this.getAll(pool);
        return storage.has(key) ? cloneDeep(storage.get(key)) : null;
    };
    MemoryStorage.prototype.put = function (_a, value) {
        var _b = _a.pool, pool = _b === void 0 ? DEFAULT_STORAGE_POOL_KEY : _b, key = _a.key;
        if (!this.storage.has(pool)) {
            this.storage.set(pool, new Map());
        }
        this.storage.get(pool).set(key, cloneDeep(value));
    };
    MemoryStorage.prototype.remove = function (_a) {
        var _b = _a.pool, pool = _b === void 0 ? DEFAULT_STORAGE_POOL_KEY : _b, key = _a.key;
        if (!key) {
            this.storage.delete(pool);
            return;
        }
        var poolStorage = this.storage.get(pool);
        if (poolStorage) {
            poolStorage.delete(key);
        }
    };
    MemoryStorage.prototype.removeAll = function () {
        this.storage = new Map();
    };
    return MemoryStorage;
}());
export { MemoryStorage };
var StorageService = (function () {
    function StorageService() {
        this.defaultStorageType = StorageType.memory;
        this.setupStorages();
    }
    StorageService.prototype.setDefaultStorageType = function (storageType) {
        this.defaultStorageType = storageType;
    };
    StorageService.prototype.getAll = function (_a) {
        var pool = _a.pool, storageType = _a.storageType;
        var storage = this.storages.get(storageType || this.defaultStorageType);
        return storage.getAll(pool);
    };
    StorageService.prototype.get = function (_a) {
        var pool = _a.pool, key = _a.key, storageType = _a.storageType;
        var data = this.storages.get(storageType || this.defaultStorageType).get({ pool: pool, key: key });
        return DataCacheStrategyFactory.getInstance().get(data);
    };
    StorageService.prototype.put = function (_a, value) {
        var pool = _a.pool, key = _a.key, storageType = _a.storageType;
        var storage = this.storages.get(storageType || this.defaultStorageType);
        return DataCacheStrategyFactory.getInstance().put({ pool: pool, key: key }, value, storage);
    };
    StorageService.prototype.remove = function (_a) {
        var pool = _a.pool, key = _a.key, storageType = _a.storageType;
        return this.storages.get(storageType || this.defaultStorageType).remove({ pool: pool, key: key });
    };
    StorageService.prototype.removeAll = function (_a) {
        var storageType = _a.storageType;
        return this.storages.get(storageType || this.defaultStorageType).removeAll();
    };
    StorageService.prototype.setupStorages = function () {
        this.storages = new Map();
        this.memoryStorage = new MemoryStorage();
        if (window) {
            this.sessionStorage = window.sessionStorage;
            this.localStorage = window.localStorage;
            this.storages.set(StorageType.memory, this.memoryStorage)
                .set(StorageType.sessionStorage, new WebStorage(this.sessionStorage))
                .set(StorageType.localStorage, new WebStorage(this.localStorage));
            return;
        }
        this.storages.set(StorageType.memory, this.memoryStorage)
            .set(StorageType.sessionStorage, this.memoryStorage)
            .set(StorageType.localStorage, this.memoryStorage);
    };
    StorageService.decorators = [
        { type: Injectable },
    ];
    StorageService.ctorParameters = function () { return []; };
    return StorageService;
}());
export { StorageService };
var StorageFactory = (function () {
    function StorageFactory() {
    }
    StorageFactory.getStorageService = function () {
        return StorageFactory.storageService;
    };
    StorageFactory.storageService = new StorageService();
    return StorageFactory;
}());
export { StorageFactory };
export function Cacheable(_a) {
    var _b = _a === void 0 ? {} : _a, _c = _b.pool, pool = _c === void 0 ? DEFAULT_STORAGE_POOL_KEY : _c, key = _b.key, _d = _b.storageType, storageType = _d === void 0 ? StorageType.memory : _d;
    var storageService = StorageFactory.getStorageService();
    var getKey = function (target, method, args) {
        var prefix = key || target.constructor.name + "." + method;
        return prefix + ":" + args.join('-');
    };
    return function (target, name, methodInfo) {
        var method = methodInfo.value;
        var proxy = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            var key = getKey(target, name, args || []);
            var data = storageService.get({ pool: pool, key: key, storageType: storageType });
            if (data) {
                return data;
            }
            var result = method.apply(this, args || []);
            return storageService.put({ pool: pool, key: key, storageType: storageType }, result);
        };
        proxy.cacheEvict = function () {
            storageService.remove({ pool: pool, key: key });
        };
        return {
            value: proxy
        };
    };
}
export var STORAGE_PROVIDERS = [
    {
        provide: StorageService,
        useClass: StorageService
    },
];
