import { ComponentFactoryResolver, Directive, ElementRef, HostBinding, Input, Optional, Renderer2, ViewContainerRef } from '@angular/core';
import { TooltipComponent } from './tooltip.component';
var TooltipDirective = (function () {
    function TooltipDirective(elementRef, hostView, resolver, renderer, tooltip) {
        this.elementRef = elementRef;
        this.hostView = hostView;
        this.resolver = resolver;
        this.renderer = renderer;
        this.isDynamicTooltip = false;
        this.tooltip = tooltip;
        if (!this.tooltip) {
            var factory = this.resolver.resolveComponentFactory(TooltipComponent);
            this.tooltip = this.hostView.createComponent(factory).instance;
            this.isDynamicTooltip = true;
        }
        this.tooltip.setOverlayOrigin(this);
    }
    Object.defineProperty(TooltipDirective.prototype, "hiTitle", {
        set: function (title) {
            if (this.isDynamicTooltip) {
                this.tooltip.hiTitle = title;
            }
        },
        enumerable: true,
        configurable: true
    });
    TooltipDirective.prototype.ngAfterViewInit = function () {
        var _this = this;
        if (this.tooltip.hiTrigger === 'hover') {
            var overlayElement_1;
            this.renderer.listen(this.elementRef.nativeElement, 'mouseenter', function () { return _this.delayEnterLeave(true, true, _this.tooltip.hiMouseEnterDelay); });
            this.renderer.listen(this.elementRef.nativeElement, 'mouseleave', function () {
                _this.delayEnterLeave(true, false, _this.tooltip.hiMouseLeaveDelay);
                if (_this.tooltip.overlay.overlayRef && !overlayElement_1) {
                    overlayElement_1 = _this.tooltip.overlay.overlayRef.overlayElement;
                    _this.renderer.listen(overlayElement_1, 'mouseenter', function () { return _this.delayEnterLeave(false, true); });
                    _this.renderer.listen(overlayElement_1, 'mouseleave', function () { return _this.delayEnterLeave(false, false); });
                }
            });
        }
        else if (this.tooltip.hiTrigger === 'focus') {
            this.renderer.listen(this.elementRef.nativeElement, 'focus', function () { return _this.show(); });
            this.renderer.listen(this.elementRef.nativeElement, 'blur', function () { return _this.hide(); });
        }
        else if (this.tooltip.hiTrigger === 'click') {
            this.renderer.listen(this.elementRef.nativeElement, 'click', function (e) {
                e.preventDefault();
                _this.show();
            });
        }
    };
    TooltipDirective.prototype.show = function () {
        this.tooltip.show();
        this.isTooltipOpen = true;
    };
    TooltipDirective.prototype.hide = function () {
        this.tooltip.hide();
        this.isTooltipOpen = false;
    };
    TooltipDirective.prototype.delayEnterLeave = function (isOrigin, isEnter, delay) {
        var _this = this;
        if (delay === void 0) { delay = -1; }
        if (this.delayTimer) {
            window.clearTimeout(this.delayTimer);
            this.delayTimer = null;
        }
        else if (delay > 0) {
            this.delayTimer = window.setTimeout(function () {
                _this.delayTimer = null;
                isEnter ? _this.show() : _this.hide();
            }, delay * 1000);
        }
        else {
            isEnter && isOrigin ? this.show() : this.hide();
        }
    };
    TooltipDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[hi-tooltip]'
                },] },
    ];
    TooltipDirective.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: ViewContainerRef, },
        { type: ComponentFactoryResolver, },
        { type: Renderer2, },
        { type: TooltipComponent, decorators: [{ type: Optional },] },
    ]; };
    TooltipDirective.propDecorators = {
        "hiTitle": [{ type: Input, args: ['hi-tooltip',] },],
        "isTooltipOpen": [{ type: HostBinding, args: ['class.hi-tooltip-open',] },],
    };
    return TooltipDirective;
}());
export { TooltipDirective };
